@@ -1,6 +1,5 @@
 """Provide functionality for TTS."""
 import asyncio
-import ctypes
 import functools as ft
 import hashlib
 import io
@@ -8,10 +7,11 @@ import logging
 import mimetypes
 import os
 import re
-from typing import Optional
+from typing import Dict, Optional
 
 from aiohttp import web
 import mutagen
+from mutagen.id3 import TextFrame as ID3Text
 import voluptuous as vol
 
 from homeassistant.components.http import HomeAssistantView
@@ -22,11 +22,18 @@ from homeassistant.components.media_player.const import (
     MEDIA_TYPE_MUSIC,
     SERVICE_PLAY_MEDIA,
 )
-from homeassistant.const import ATTR_ENTITY_ID, CONF_PLATFORM
+from homeassistant.const import (
+    ATTR_ENTITY_ID,
+    CONF_PLATFORM,
+    HTTP_BAD_REQUEST,
+    HTTP_NOT_FOUND,
+    HTTP_OK,
+)
 from homeassistant.core import callback
 from homeassistant.exceptions import HomeAssistantError
 from homeassistant.helpers import config_per_platform, discovery
 import homeassistant.helpers.config_validation as cv
+from homeassistant.helpers.network import get_url
 from homeassistant.helpers.typing import HomeAssistantType
 from homeassistant.setup import async_prepare_setup_platform
 
@@ -40,6 +47,8 @@ ATTR_MESSAGE = "message"
 ATTR_OPTIONS = "options"
 ATTR_PLATFORM = "platform"
 
+BASE_URL_KEY = "tts_base_url"
+
 CONF_BASE_URL = "base_url"
 CONF_CACHE = "cache"
 CONF_CACHE_DIR = "cache_dir"
@@ -108,11 +117,12 @@ async def async_setup(hass, config):
         use_cache = conf.get(CONF_CACHE, DEFAULT_CACHE)
         cache_dir = conf.get(CONF_CACHE_DIR, DEFAULT_CACHE_DIR)
         time_memory = conf.get(CONF_TIME_MEMORY, DEFAULT_TIME_MEMORY)
-        base_url = conf.get(CONF_BASE_URL) or hass.config.api.base_url
+        base_url = conf.get(CONF_BASE_URL) or get_url(hass)
+        hass.data[BASE_URL_KEY] = base_url
 
         await tts.async_init_cache(use_cache, cache_dir, time_memory, base_url)
-    except (HomeAssistantError, KeyError) as err:
-        _LOGGER.error("Error on cache init %s", err)
+    except (HomeAssistantError, KeyError):
+        _LOGGER.exception("Error on cache init")
         return False
 
     hass.http.register_view(TextToSpeechView(tts))
@@ -169,7 +179,11 @@ async def async_setup(hass, config):
             }
 
             await hass.services.async_call(
-                DOMAIN_MP, SERVICE_PLAY_MEDIA, data, blocking=True
+                DOMAIN_MP,
+                SERVICE_PLAY_MEDIA,
+                data,
+                blocking=True,
+                context=service.context,
             )
 
         service_name = p_config.get(CONF_SERVICE_NAME, f"{p_type}_{SERVICE_SAY}")
@@ -205,6 +219,16 @@ async def async_setup(hass, config):
     return True
 
 
+def _hash_options(options: Dict) -> str:
+    """Hashes an options dictionary."""
+    opts_hash = hashlib.blake2s(digest_size=5)
+    for key, value in sorted(options.items()):
+        opts_hash.update(str(key).encode())
+        opts_hash.update(str(value).encode())
+
+    return opts_hash.hexdigest()
+
+
 class SpeechManager:
     """Representation of a speech store."""
 
@@ -231,14 +255,14 @@ class SpeechManager:
                 _init_tts_cache_dir, self.hass, cache_dir
             )
         except OSError as err:
-            raise HomeAssistantError(f"Can't init cache dir {err}")
+            raise HomeAssistantError(f"Can't init cache dir {err}") from err
 
         try:
             cache_files = await self.hass.async_add_executor_job(
                 _get_cache_files, self.cache_dir
             )
         except OSError as err:
-            raise HomeAssistantError(f"Can't read cache dir {err}")
+            raise HomeAssistantError(f"Can't read cache dir {err}") from err
 
         if cache_files:
             self.file_cache.update(cache_files)
@@ -297,11 +321,13 @@ class SpeechManager:
             ]
             if invalid_opts:
                 raise HomeAssistantError(f"Invalid options found: {invalid_opts}")
-            options_key = ctypes.c_size_t(hash(frozenset(options))).value
+            options_key = _hash_options(options)
         else:
             options_key = "-"
 
-        key = KEY_PATTERN.format(msg_hash, language, options_key, engine).lower()
+        key = KEY_PATTERN.format(
+            msg_hash, language.replace("_", "-"), options_key, engine
+        ).lower()
 
         # Is speech already in memory
         if key in self.mem_cache:
@@ -332,9 +358,14 @@ class SpeechManager:
         # Create file infos
         filename = f"{key}.{extension}".lower()
 
-        data = self.write_tags(filename, data, provider, message, language, options)
+        # Validate filename
+        if not _RE_VOICE_FILE.match(filename):
+            raise HomeAssistantError(
+                f"TTS filename '{filename}' from {engine} is invalid!"
+            )
 
         # Save to memory
+        data = self.write_tags(filename, data, provider, message, language, options)
         self._async_store_to_memcache(key, filename, data)
 
         if cache:
@@ -378,9 +409,9 @@ class SpeechManager:
 
         try:
             data = await self.hass.async_add_executor_job(load_speech)
-        except OSError:
+        except OSError as err:
             del self.file_cache[key]
-            raise HomeAssistantError(f"Can't read {voice_file}")
+            raise HomeAssistantError(f"Can't read {voice_file}") from err
 
         self._async_store_to_memcache(key, filename, data)
 
@@ -392,7 +423,7 @@ class SpeechManager:
         @callback
         def async_remove_from_mem():
             """Cleanup memcache."""
-            self.mem_cache.pop(key)
+            self.mem_cache.pop(key, None)
 
         self.hass.loop.call_later(self.time_memory, async_remove_from_mem)
 
@@ -431,16 +462,15 @@ class SpeechManager:
         album = provider.name
         artist = language
 
-        if options is not None:
-            if options.get("voice") is not None:
-                artist = options.get("voice")
+        if options is not None and options.get("voice") is not None:
+            artist = options.get("voice")
 
         try:
             tts_file = mutagen.File(data_bytes, easy=True)
             if tts_file is not None:
-                tts_file["artist"] = artist
-                tts_file["album"] = album
-                tts_file["title"] = message
+                tts_file["artist"] = ID3Text(encoding=3, text=artist)
+                tts_file["album"] = ID3Text(encoding=3, text=album)
+                tts_file["title"] = ID3Text(encoding=3, text=message)
                 tts_file.save(data_bytes)
         except mutagen.MutagenError as err:
             _LOGGER.error("ID3 tag error: %s", err)
@@ -507,7 +537,7 @@ def _get_cache_files(cache_dir):
         record = _RE_VOICE_FILE.match(file_data)
         if record:
             key = KEY_PATTERN.format(
-                record.group(1), record.group(2), record.group(3), record.group(4),
+                record.group(1), record.group(2), record.group(3), record.group(4)
             )
             cache[key.lower()] = file_data.lower()
     return cache
@@ -524,14 +554,16 @@ class TextToSpeechUrlView(HomeAssistantView):
         """Initialize a tts view."""
         self.tts = tts
 
-    async def post(self, request):
+    async def post(self, request: web.Request) -> web.Response:
         """Generate speech and provide url."""
         try:
             data = await request.json()
         except ValueError:
-            return self.json_message("Invalid JSON specified", 400)
+            return self.json_message("Invalid JSON specified", HTTP_BAD_REQUEST)
         if not data.get(ATTR_PLATFORM) and data.get(ATTR_MESSAGE):
-            return self.json_message("Must specify platform and message", 400)
+            return self.json_message(
+                "Must specify platform and message", HTTP_BAD_REQUEST
+            )
 
         p_type = data[ATTR_PLATFORM]
         message = data[ATTR_MESSAGE]
@@ -543,10 +575,10 @@ class TextToSpeechUrlView(HomeAssistantView):
             url = await self.tts.async_get_url(
                 p_type, message, cache=cache, language=language, options=options
             )
-            resp = self.json({"url": url}, 200)
+            resp = self.json({"url": url}, HTTP_OK)
         except HomeAssistantError as err:
             _LOGGER.error("Error on init tts: %s", err)
-            resp = self.json({"error": err}, 400)
+            resp = self.json({"error": err}, HTTP_BAD_REQUEST)
 
         return resp
 
@@ -562,12 +594,17 @@ class TextToSpeechView(HomeAssistantView):
         """Initialize a tts view."""
         self.tts = tts
 
-    async def get(self, request, filename):
+    async def get(self, request: web.Request, filename: str) -> web.Response:
         """Start a get request."""
         try:
             content, data = await self.tts.async_read_tts(filename)
         except HomeAssistantError as err:
             _LOGGER.error("Error on load tts: %s", err)
-            return web.Response(status=404)
+            return web.Response(status=HTTP_NOT_FOUND)
 
         return web.Response(body=data, content_type=content)
+
+
+def get_base_url(hass):
+    """Get base URL."""
+    return hass.data[BASE_URL_KEY]
